package cn.estsh.i3plus.ext.mes.pcn.pojo.context;

import cn.estsh.i3plus.pojo.mes.bean.MesWorkCenter;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import com.alibaba.fastjson.JSONObject;
import io.swagger.annotations.ApiParam;
import lombok.Data;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 生产过程上下文对象-产品加工规则
 */
@Data
public class MesProdRuleContext implements Serializable {

    private static final long serialVersionUID = 1522652732930288854L;

    @ApiParam("组织代码")
    public String organizeCode;

    @ApiParam(name = "数据关联键")
    private Integer foreignKey;

    @ApiParam("装配件绑定清单  [JSON]List<MesProductionAssemblyContext>")
    public String assemblyDataJson;

    //-------以下排序属性-------------

    @ApiParam("生产线代码")
    public String workCenterCode;

    @ApiParam("工位代码")
    public String workCellCode;

    @ApiParam("工序代码")
    public String processCode;

    //-------以下非排序属性-------------

    @ApiParam(name = "读/扫主条码信息(内制件/外协件),不一定有值")
    private String productSn;

    @ApiParam("非排序产品加工规则ID")
    public Long pid;

    @ApiParam(name = "进料零件号")
    private String inPartNo;

    @ApiParam(name = "产成零件号")
    private String outPartNo;

    @ApiParam(name = "设备代码")
    private String equipmentCode;

    @ApiParam(name = "进料零件规则")
    private String inPartNoRule;

    @ApiParam(name = "是否打印条码")
    private Integer isPrint;

    @ApiParam(name = "是否有装配件")
    private Integer isHasBind;
    /**
     * 非排序报工方式 只有 汇报 或者 移库 二选一
     */
    @ApiParam(name = "汇报方式")
    private Integer reportType;

    @ApiParam(name = "是否验证加工单数量")
    private Integer isCheckOrderQty;

    @ApiParam(name = "是否包装")
    private Integer isPackage;

    @ApiParam(name = "是否验证装配件扫描顺序")
    private Integer isCheckBindSeq;

    @ApiParam(name = "成品站点")
    private String partLocation;

    @ApiParam(name = "子箱设备信号点")
    private String packageEquSignal;

    //----------仅作为查询条件冗余-------------

    @ApiParam(name = "进料零件号与规则是否均为空")
    private Boolean inPartIsEmpty = false;

    @ApiParam(name = "进料规则是否有值")
    private Boolean inPartRuleIsNotEmpty = false;

    @ApiParam(name = "进料零件号")
    private String inPartNos;

    @ApiParam(name = "产成零件号")
    private String outPartNos;

    public MesProdRuleContext() {}

    public MesProdRuleContext(String organizeCode) {
        this.organizeCode = organizeCode;
    }

    public MesProdRuleContext(String organizeCode, String workCenterCode, String workCellCode, String processCode) {
        this.organizeCode = organizeCode;
        this.workCenterCode = workCenterCode;
        this.workCellCode = workCellCode;
        this.processCode = processCode;
    }

    public MesProdRuleContext foreignKey(Integer foreignKey) {
        this.foreignKey = foreignKey;
        return this;
    }

    public MesProdRuleContext inPartIsEmpty() {
        this.inPartIsEmpty = true;
        return this;
    }

    public MesProdRuleContext inPartRuleIsNotEmpty() {
        this.inPartRuleIsNotEmpty = true;
        return this;
    }

    public MesProdRuleContext pid(Long pid) {
        this.pid = pid;
        return this;
    }

    public MesProdRuleContext inPartNo(String inPartNo) {
        this.inPartNo = inPartNo;
        return this;
    }

    public MesProdRuleContext outPartNo(String outPartNo) {
        this.outPartNo = outPartNo;
        return this;
    }

    public MesProdRuleContext inPartNos(List<String> inPartNoList) {
        if (CollectionUtils.isEmpty(inPartNoList)) return this;
        inPartNoList = inPartNoList.stream().filter(o -> !StringUtils.isEmpty(o)).distinct().collect(Collectors.toList());
        if (inPartNoList.size() > 1) this.inPartNos = JSONObject.toJSONString(inPartNoList);
        else this.inPartNo = inPartNoList.get(0);
        return this;
    }

    public MesProdRuleContext outPartNos(List<String> outPartNoList) {
        if (CollectionUtils.isEmpty(outPartNoList)) return this;
        outPartNoList = outPartNoList.stream().filter(o -> !StringUtils.isEmpty(o)).distinct().collect(Collectors.toList());
        if (outPartNoList.size() > 1) this.outPartNos = JSONObject.toJSONString(outPartNoList);
        else this.outPartNo = outPartNoList.get(0);
        return this;
    }

    public List<String> getInPartNoList() {
        return !StringUtils.isEmpty(this.inPartNos) ? JSONObject.parseObject(this.inPartNos, List.class) : null;
    }

    public List<String> getOutPartNoList() {
        return !StringUtils.isEmpty(this.outPartNos) ? JSONObject.parseObject(this.outPartNos, List.class) : null;
    }

    public MesProdRuleContext equipmentCode(String equipmentCode) {
        this.equipmentCode = equipmentCode;
        return this;
    }

    public MesProdRuleContext assemblyDataJson(List<? extends MesProductionAssemblyContext> productionAssemblyContextList) {
        this.assemblyDataJson = !CollectionUtils.isEmpty(productionAssemblyContextList) ? JSONObject.toJSONString(productionAssemblyContextList) : null;
        return this;
    }

    //根据生产线类型返回装配件数据集合【向上转型】
    public List<MesProductionAssemblyContext> getAssemblyDataContext(MesWorkCenter workCenter) {
        if (StringUtils.isEmpty(this.assemblyDataJson)) return null;
        List<MesProductionAssemblyContext> productionAssemblyContextList = new ArrayList<>();
        if (MesExtEnumUtil.WORK_CENTER_TYPE.SORT.getValue() == workCenter.getCenterType()) {
            List<MesProductionAssemblySortContext> productionAssemblySortContextList = JSONObject.parseArray(this.assemblyDataJson, MesProductionAssemblySortContext.class);
            productionAssemblySortContextList.forEach(o -> productionAssemblyContextList.add(o));
        } else {
            List<MesProductionAssemblyNosortContext> productionAssemblyNosortContextList = JSONObject.parseArray(this.assemblyDataJson, MesProductionAssemblyNosortContext.class);
            productionAssemblyNosortContextList.forEach(o -> productionAssemblyContextList.add(o));
        }
        return productionAssemblyContextList;
    }

}
